@@ -6,16 +6,17 @@ use parent 'Net::OAuth2Server::Request';
 sub allowed_methods       { 'POST' }
 sub accepted_auth         { 'Basic' }
 sub required_parameters   { 'grant_type' }
-sub required_confidential { 'client_secret' }
+sub confidential_parameters { 'client_secret' }
 
 sub get_grant {}
 
 sub dispatch {
 	my ( $self, @class ) = ( shift, @_ );
 	return $self if $self->error;
-	my %grant_type_class = map { s/\A(\+?)/__PACKAGE__.'::' x !$1/e unless ref; ( $_->grant_type, $_ ) } @class;
-	my $class = $grant_type_class{ $self->param( 'grant_type' ) };
-	$class ? $class->new( %$self ) : $self->with_error_unsupported_grant_type;
+	for ( @class ) { s/\A\+/__PACKAGE__.'::'/e unless ref }
+	my $type = $self->param( 'grant_type' );
+	my ( $class ) = defined $type ? grep $type eq $_->grant_type, @class : ();
+	$class ? $class->new( %$self ) : $self->set_error_unsupported_grant_type;
 }
 
-our $VERSION = '0.002';
+our $VERSION = '0.003';
